package ms.warpzone.warppay.manager;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.util.Log;

import com.activeandroid.query.Select;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.StringTokenizer;

import ms.warpzone.warppay.MainActivity;
import ms.warpzone.warppay.R;
import ms.warpzone.warppay.data.SQLiteService;
import ms.warpzone.warppay.data.models.local.Product;
import ms.warpzone.warppay.data.models.local.User;
import ms.warpzone.warppay.data.models.rest.RestProduct;
import ms.warpzone.warppay.data.models.rest.RestTransaction;

import ms.warpzone.warppay.data.models.rest.RestUser;
import ms.warpzone.warppay.dialogs.PinCodesDialog;
import ms.warpzone.warppay.orderList.Order;
import retrofit.Callback;
import retrofit.Response;
import retrofit.Retrofit;

public class MainManager {
    private static MainManager instance = new MainManager();

    private MainActivity mainActivity;
    private SQLiteService sqLiteService;
    private UiManager uiManager;
    private DataManager dataManager;
    private boolean barcodeLearning=false;



    public static MainManager getInstance() {
        return instance;
    }

    private MainManager() {
    }

    public void init(MainActivity mainActivity) {
        this.mainActivity = mainActivity;
        this.sqLiteService = SQLiteService.getInstance();
        this.uiManager = UiManager.getInstance();
        this.dataManager = DataManager.getInstance();
        this.uiManager.initUi(this.mainActivity);
        RestManager restManager = RestManager.getInstance();
        restManager.initRestService();

        RefreshManager.getInstance().refreshData();
        RefreshManager.getInstance().startRefreshTimer();

    }


    SQLiteService getSqLiteService() {
        return sqLiteService;
    }

    void setCurrentUser(User user) {
        this.setCurrentUser(user,false);
    }

    public void setCurrentUser(final User user, boolean authorized) {
        if (user != null) {
            if(!user.getPinCode().isEmpty() && !authorized) {
                PinCodesDialog p = new PinCodesDialog(user);
                p.setOnDismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialogInterface) {
                        if(MainManager.getInstance().getCurrentUser() == null)
                            MainManager.getInstance().clearCurrentUser();
                    }
                });
                p.show();
                return;
            }
            if (user.getUserid() != "Guest") {
                final ProgressDialog pd = ProgressDialog.show(MainManager.getInstance().getMainActivity(), "Processing...", "Loading...");
                RestManager.getInstance().getRestService().getSingleUser(user.getUserid()).enqueue(new Callback<RestUser>() {
                    @Override
                    public void onResponse(Response<RestUser> response, Retrofit retrofit) {
                        if (response.code() == 200) {
                            user.setCredit(response.body().getCredit());
                            user.save();
                            DataManager.getInstance().setCurrentUser(user);
                            UiManager.getInstance().setCurrentUser(user);
                            pd.dismiss();

                        } else {
                            pd.dismiss();
                            Log.d("REST_3", String.valueOf(response.code()));
                            UiManager.getInstance().showWarning("Transaction Error. Please try again", 4000);
                        }
                    }

                    @Override
                    public void onFailure(Throwable t) {
                        Log.d("REST_4", String.valueOf(t.getMessage()));
                        UiManager.getInstance().showWarning("Transaction Error. Please try again", 4000);

                    }
                });
            } else {
                DataManager.getInstance().setCurrentUser(user);
                UiManager.getInstance().setCurrentUser(user);
            }

            RefreshManager.getInstance().startTimer();
            RefreshManager.getInstance().stopRefreshTimer();
        }
    }


    public User getCurrentUser() {
        return this.dataManager.getCurrentUser();
    }

    void clearCurrentUser() {
        DataManager.getInstance().setLastCardId("");
        this.dataManager.clearCurrentUser();
        this.uiManager.clearCurrentUser();
    }

    public void addOrder(Order order) {
        DataManager.getInstance().addOrder(order);
        this.uiManager.refreshTotalTextView(DataManager.getInstance().getTotalAmount());
    }

    public void removeOrder(Order order) {
        DataManager.getInstance().removeOrder(order);
        this.uiManager.removeOrder(order);
        this.uiManager.refreshTotalTextView(DataManager.getInstance().getTotalAmount());

    }


    public void chargeAmount(BigDecimal amount) {
        User currentUser = this.dataManager.getCurrentUser();
        if(amount.compareTo(new BigDecimal(0)) < 0 || amount.scale() > 2) {
            UiManager.getInstance().showWarning("Fehlerhafter Betrag angegeben.",4000);
            return;
        }

        if(currentUser.getCredit().add(amount).compareTo(new BigDecimal(50.0)) > 0) {
            UiManager.getInstance().showWarning("Maximal 50 Euro Guthaben erlaubt!",4000);
            return;
        }

        RestTransaction t = new RestTransaction();
        t.setAmount(amount);
        t.setTrans_type(1);
        t.setCash_paid(true);
        ArrayList<RestTransaction> transactions = new ArrayList<>();
        transactions.add(t);

        TransactionManager.getInstance().perform_transactions(transactions,true,false);
    }

    public void performPayment(Boolean cash) {
        User currentUser = DataManager.getInstance().getCurrentUser();
        BigDecimal totalAmount = DataManager.getInstance().getTotalAmount();

        if (totalAmount.compareTo(new BigDecimal(0.0)) > 0 && currentUser != null){
            if (currentUser.getCredit().compareTo(totalAmount) >= 0 || cash){

                ArrayList<RestTransaction> transactions = new ArrayList<>();

                for (Order order:DataManager.getInstance().getOrderList()) {
                    RestTransaction transaction = new RestTransaction();
                    transaction.setProduct(RestProduct.fromLocalProduct(order.getProduct()));
                    transaction.setTrans_type(2);
                    if (DataManager.getInstance().getIs_guest() || cash)
                        transaction.setCash_paid(true);
                    transactions.add(transaction);
                }

                TransactionManager.getInstance().perform_transactions(transactions, !cash, true);

            } else {
                UiManager.getInstance().showWarning(mainActivity.getResources().getString(R.string.please_charge), 2500);
            }
        }
    }


    public MainActivity getMainActivity() {
        return mainActivity;
    }

    public void saveCardId() {
        final String card_id = this.dataManager.getLastCardId();

        if (card_id != null) {
            new AlertDialog.Builder(this.mainActivity)
                .setTitle(this.mainActivity.getResources().getString(R.string.confirm_learn_card))
                .setMessage("Die Karten-ID "+card_id+" wird dem Nutzer "+this.getCurrentUser()+" hinzugefügt")
                .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        MainManager.getInstance().getCurrentUser().setCardId(card_id);
                        MainManager.getInstance().dataManager.saveCurrentUser();

                    }
                })
                .setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                    }
                })
                .setIcon(android.R.drawable.ic_dialog_alert)
                .show();
        }
    }

    public void setBarcodeLearning(boolean barcodeLearning) {
        this.barcodeLearning = barcodeLearning;
    }
}
