package ms.warpzone.warppay.manager;

import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.os.Handler;
import android.util.Log;

import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import ms.warpzone.warppay.data.models.local.Category;
import ms.warpzone.warppay.data.models.local.User;
import ms.warpzone.warppay.data.models.rest.RestCategory;
import ms.warpzone.warppay.data.models.rest.RestProduct;
import ms.warpzone.warppay.data.models.rest.RestUser;
import ms.warpzone.warppay.dialogs.NotificationDialog;
import ms.warpzone.warppay.dialogs.NyanCatDialog;
import retrofit.Callback;
import retrofit.Response;
import retrofit.Retrofit;

public class RefreshManager {
    private static RefreshManager ourInstance = new RefreshManager();
    private Timer timer;
    private Timer screenSaverTimer;
    private Timer tmrRefresh;
    public ProgressDialog pdProcess;
    private NotificationDialog ndRefreshError = null;

    private RefreshManager() {}
    public static RefreshManager getInstance() {
        return ourInstance;
    }

    void refreshData() {
        this.refreshData(false);
    }

    public void refreshData(boolean disableDialog) {
        if(this.ndRefreshError != null)
            this.ndRefreshError.dismiss();
        this.ndRefreshError = null;

        if(!disableDialog)
            pdProcess = ProgressDialog.show(MainManager.getInstance().getMainActivity(), "Processing...", "Loading...");



        RestManager.getInstance().getRestService().getAllUser().enqueue(new Callback<List<RestUser>>() {
            @Override
            public void onResponse(Response<List<RestUser>> response, Retrofit retrofit) {
                if(response.code() == 200 ) {
                    List<User> userList = MainManager.getInstance().getSqLiteService().refreshUserData(response.body());
                    if (userList != null) {
                        UiManager.getInstance().refreshUserData(userList);
                    }
                } else {
                    RefreshManager.getInstance().showRefreshError();
                }
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
                RefreshManager.getInstance().showRefreshError();
            }
        });

        RestManager.getInstance().getRestService().getAllCategories().enqueue(new Callback<List<RestCategory>>() {
            @Override
            public void onResponse(Response<List<RestCategory>> response, Retrofit retrofit) {
                if(response.code() == 200 ) {
                    List<Category> categoryList = MainManager.getInstance().getSqLiteService().refreshCategoryData(response.body());
                    if (categoryList != null) {
                        UiManager.getInstance().showCategoryButtons();
                    }
                } else {
                    RefreshManager.getInstance().showRefreshError();
                }
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
                RefreshManager.getInstance().showRefreshError();
            }
        });

        RestManager.getInstance().getRestService().getAllProducts().enqueue(new Callback<List<RestProduct>>() {
            @Override
            public void onResponse(Response<List<RestProduct>> response, Retrofit retrofit) {
                if(response.code() == 200 ) {
                    List<ms.warpzone.warppay.data.models.local.Product> productList = MainManager.getInstance().getSqLiteService().refreshProductData(response.body());
                    if (productList != null) {
                        UiManager.getInstance().refreshProductData(productList);
                    }
                    if(RefreshManager.getInstance().pdProcess != null ) {
                        RefreshManager.getInstance().pdProcess.dismiss();
                        RefreshManager.getInstance().pdProcess = null;
                    }
                } else {
                    RefreshManager.getInstance().showRefreshError();
                }
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
                RefreshManager.getInstance().showRefreshError();
            }
        });
    }

    public void showRefreshError() {
        if(ndRefreshError == null) {
            RefreshManager.getInstance().stopRefreshTimer();
            if (RefreshManager.getInstance().pdProcess != null) {
                RefreshManager.getInstance().pdProcess.dismiss();
                RefreshManager.getInstance().pdProcess = null;
            }
            ndRefreshError = new NotificationDialog(MainManager.getInstance().getMainActivity(),"WARNING","Refresh failed. Please retry!");
            ndRefreshError.setWarning();
            ndRefreshError.getBuilder().setNeutralButton("Refresh", new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialogInterface, int i) {
                    RefreshManager.getInstance().refreshData();

                }
            });
            ndRefreshError.show();

            final Handler handler = new Handler();
            final Runnable runnable = new Runnable() {
                @Override
                public void run() {
                    RefreshManager.getInstance().refreshData();
                }
            };

            ndRefreshError.getDialog().setOnDismissListener(new DialogInterface.OnDismissListener() {
                @Override
                public void onDismiss(DialogInterface dialog) {
                    handler.removeCallbacks(runnable);
                }
            });

            handler.postDelayed(runnable, 10000);
        }

    }
    public void stopTimer() {
        if (this.timer != null) {
            this.timer.cancel();
            this.timer.purge();
        }
    }

    public void startTimer() {
        this.stopTimer();
        this.timer = new Timer();
        TimerTask t = new TimerTask() {
            @Override
            public void run() {
                Handler mainHandler = new Handler(MainManager.getInstance().getMainActivity().getApplicationContext().getMainLooper());
                Runnable myRunnable = new Runnable() {
                    @Override
                    public void run() {
                        MainManager.getInstance().clearCurrentUser();
                    }
                };
                mainHandler.post(myRunnable);
            }
        };

        this.timer.schedule(t, 60000);
    }

    public void startRefreshTimer() {
        this.tmrRefresh = new Timer();
        TimerTask t = new TimerTask() {
            @Override
            public void run() {
                Handler mainHandler = new Handler(MainManager.getInstance().getMainActivity().getApplicationContext().getMainLooper());
                Runnable myRunnable = new Runnable() {
                    @Override
                    public void run() {

                        Log.d("REST","REFRESH TIMER RUNNING");
                        if(MainManager.getInstance().getCurrentUser() == null)
                            RefreshManager.getInstance().refreshData(true);
                    }
                };
                mainHandler.post(myRunnable);
            }
        };
        Log.d("REST","STARTING REFRESH TIMER");
        this.tmrRefresh.schedule(t, 60000,60000);
    }
    public void stopRefreshTimer() {
        if (this.tmrRefresh != null) {
            this.tmrRefresh.cancel();
            this.tmrRefresh.purge();
        }
    }
    public void resetScreenSaverTimer() {
        this.stopScreenSaverTimer();
        this.screenSaverTimer = new Timer();

        TimerTask t = new TimerTask() {
            @Override
            public void run() {
                Handler mainHandler = new Handler(MainManager.getInstance().getMainActivity().getApplicationContext().getMainLooper());
                Runnable myRunnable = new Runnable() {
                    @Override
                    public void run() {
                        new NyanCatDialog().show();
                    }
                };
                mainHandler.post(myRunnable);
            }
        };
        this.screenSaverTimer.schedule(t, 4000);
    }

    public void stopScreenSaverTimer() {
        if (this.screenSaverTimer != null) {
            this.screenSaverTimer.cancel();
            this.screenSaverTimer.purge();
        }
    }

}
