package ms.warpzone.warppay.manager;


import android.util.Log;

import java.util.ArrayList;

import ms.warpzone.warppay.data.models.local.User;
import ms.warpzone.warppay.data.models.rest.RestTransaction;
import ms.warpzone.warppay.data.models.rest.RestUser;
import retrofit.Callback;
import retrofit.Response;
import retrofit.Retrofit;

public class TransactionManager {
    private static TransactionManager ourInstance = new TransactionManager();


    private TransactionManager() {}
    public static TransactionManager getInstance() {
        return ourInstance;
    }

    /***
     * Perform a Transaction and update the User.
     * @param transactions
     * @return
     */
    public void perform_transactions(ArrayList<RestTransaction> transactions, final boolean show_credit_dialog, final boolean logout) {
        User currentUser = DataManager.getInstance().getCurrentUser();

        RestManager.getInstance().getRestService().saveTransaction(currentUser.getUserid(), transactions).enqueue(new Callback<Void>() {
            @Override
            public void onResponse(Response<Void> response, Retrofit retrofit) {
                if(response.code() != 200)  {
                    Log.d("REST_1", String.valueOf(response.code()));
                    UiManager.getInstance().showWarning("Transaction Error. Please try Again", 4000);
                }

            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
                UiManager.getInstance().showWarning("Transaction Error. Please try Again", 4000);
            }
        });
        if(currentUser.getUserid() == "Guest") {
            if (logout)
                MainManager.getInstance().clearCurrentUser();
            return;
        }
        RestManager.getInstance().getRestService().getSingleUser(currentUser.getUserid()).enqueue(new Callback<RestUser>() {
            @Override
            public void onResponse(Response<RestUser> response, Retrofit retrofit) {
                if(response.code() == 200) {
                    DataManager.getInstance().getCurrentUser().setCredit(response.body().getCredit());
                    UiManager.getInstance().refreshCreditTextView(DataManager.getInstance().getCurrentUser().getCredit());
                    DataManager.getInstance().getCurrentUser().save();
                    if (show_credit_dialog) {
                        UiManager.getInstance().showMessage("Neues Guthaben: " + DataManager.getInstance().getCurrentUser().getCredit() + " Euro", 2500);
                    }
                    if (logout)
                        MainManager.getInstance().clearCurrentUser();
                } else {
                    Log.d("REST_2", String.valueOf(response.code()));
                    UiManager.getInstance().showWarning("Transaction Error. Please try again", 4000);
                }
            }

            @Override
            public void onFailure(Throwable t) {
                UiManager.getInstance().showWarning("Transaction Error. Please try again", 4000);
                Log.d("REST", t.getMessage());

            }
        });
    }

}
