package ms.warpzone.warppay.manager;

import android.annotation.TargetApi;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.os.Build;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.GridLayout;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.PopupMenu;
import android.widget.TextView;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import ms.warpzone.warppay.MainActivity;
import ms.warpzone.warppay.R;
import ms.warpzone.warppay.data.models.local.Category;
import ms.warpzone.warppay.data.models.local.Product;
import ms.warpzone.warppay.data.models.local.User;
import ms.warpzone.warppay.dialogs.ChargeCustomDialog;
import ms.warpzone.warppay.dialogs.ChargeDialog;
import ms.warpzone.warppay.dialogs.NotificationDialog;
import ms.warpzone.warppay.dialogs.PayChoiceDialog;
import ms.warpzone.warppay.dialogs.PinCodesDialog;
import ms.warpzone.warppay.dialogs.ProductDialog;
import ms.warpzone.warppay.listener.CategoryOnClickListener;
import ms.warpzone.warppay.orderList.ListViewAdapter;
import ms.warpzone.warppay.orderList.ListViewAdapterProducts;
import ms.warpzone.warppay.orderList.Order;

public class UiManager implements View.OnKeyListener, View.OnClickListener, AdapterView.OnItemClickListener {

    private MainActivity mainActivity;
    private Button btnPay, btnCharge, btnLogout, btnGuest, btnSettings;
    private ImageButton btnRefresh;
    private TextView txtSum,txtCredit;
    private ListView lstOrdered;
    private GridLayout gridCategories;
    private AutoCompleteTextView atxvName;

    private ListViewAdapter lstOrderedAdapter;
    private ListViewAdapterProducts lstProductAdapter;
    private ArrayAdapter atxvAdapter;
    private ArrayList<Double> order;
    private List<User> users;
    private ProgressDialog processDialog = null;

    private PayChoiceDialog payChoiceDialog;
    private ProductDialog productDialog;
    private ChargeDialog chargeDialog;
    private ChargeCustomDialog chargeCustomDialog;
    private PinCodesDialog pinCodesDialog;
    private PopupMenu popupMenuSettings;
    private static UiManager ourInstance = new UiManager();

    public static UiManager getInstance() {
        return ourInstance;
    }

    private UiManager() {}

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    protected void initUi(MainActivity mainActivity) {
        this.txtSum = (TextView) mainActivity.findViewById(R.id.txtSum);
        this.txtCredit = (TextView)mainActivity.findViewById(R.id.txtCredit);
        this.txtCredit.setText(mainActivity.getResources().getString(R.string.credit,"0.0"));
        this.txtSum.setText(mainActivity.getResources().getString(R.string.total,"0.0"));

        this.btnLogout = (Button) mainActivity.findViewById(R.id.btnLogout);
        this.btnRefresh = (ImageButton) mainActivity.findViewById(R.id.btnRefresh);
        this.btnPay = (Button) mainActivity.findViewById(R.id.btnPay);
        this.btnCharge = (Button) mainActivity.findViewById(R.id.btnCharge);
        this.btnGuest = (Button) mainActivity.findViewById(R.id.btnGuest);
        this.btnSettings = (Button) mainActivity.findViewById(R.id.btnSettings);

        this.lstOrderedAdapter = new ListViewAdapter(mainActivity, R.layout.order_list_item, new ArrayList<Order>());
        this.lstProductAdapter = new ListViewAdapterProducts(mainActivity, R.layout.product_list_item, Product.getAll());

        this.lstOrdered = (ListView) mainActivity.findViewById(R.id.lstOrdered);
        this.lstOrdered.setAdapter(this.lstOrderedAdapter);

        this.users = User.getAll();
        this.atxvAdapter = new ArrayAdapter(mainActivity, android.R.layout.select_dialog_item, this.users);
        this.atxvName = (AutoCompleteTextView) mainActivity.findViewById(R.id.atxvName);
        this.atxvName.setAdapter(atxvAdapter);
        this.atxvName.setThreshold(1);
        this.atxvName.setOnKeyListener(this);
        this.atxvName.setOnItemClickListener(this);
        this.atxvName.setFocusableInTouchMode(true);
        this.atxvName.setFocusable(true);
        this.atxvName.requestFocus();

        this.gridCategories = (GridLayout) mainActivity.findViewById(R.id.gridCategories);
        this.btnLogout.setVisibility(View.INVISIBLE);
        this.enableControls(false);
        this.mainActivity = mainActivity;

    }

    public void showSoftKeyboard(View view) {
        if (view.requestFocus()) {
            InputMethodManager imm = (InputMethodManager)
                    this.mainActivity.getSystemService(this.mainActivity.getApplicationContext().INPUT_METHOD_SERVICE);
            imm.showSoftInput(view, InputMethodManager.SHOW_FORCED);
        }
    }
    protected void refreshTotalTextView(BigDecimal totalAmount) {
        this.txtSum.setText(this.mainActivity.getResources().getString(R.string.total,totalAmount.toString()));
    }
    protected void refreshCreditTextView(BigDecimal credit) {
        this.txtCredit.setText(this.mainActivity.getResources().getString(R.string.credit, credit.toString()));
    }

    protected void setCurrentUser(User user) {
        this.enableControls(true);
        this.atxvName.setText(user.getUserid());
        this.atxvName.dismissDropDown();
        this.btnLogout.setVisibility(View.VISIBLE);
        this.refreshCreditTextView(user.getCredit());
    }

    protected void clearCurrentUser() {
        this.atxvName.setText("");

        this.lstOrderedAdapter.clear();
        this.enableControls(false);
        this.btnLogout.setVisibility(View.INVISIBLE);
        this.refreshCreditTextView(new BigDecimal(0.0));
        this.refreshTotalTextView(new BigDecimal(0.0));
        if(this.atxvName.requestFocus()) {
            this.showSoftKeyboard(this.mainActivity.getCurrentFocus());
        }

        if(this.payChoiceDialog != null)
            this.payChoiceDialog.dismiss();
        if(this.chargeCustomDialog != null)
            this.chargeCustomDialog.dismiss();
        if(this.chargeDialog != null)
            this.chargeDialog.dismiss();
        if(this.productDialog != null)
            this.productDialog.dismiss();
        if(this.popupMenuSettings != null)
            this.popupMenuSettings.dismiss();
    }

    private void enableControls(boolean enable) {
        int visible;
        if(enable)
            visible= View.VISIBLE;
        else
            visible= View.INVISIBLE;
        this.atxvName.setEnabled(!enable);
        this.btnRefresh.setEnabled(!enable);
        this.btnLogout.setEnabled(enable);
        this.gridCategories.setVisibility(visible);
        this.btnPay.setEnabled(enable);
        this.btnPay.setVisibility(visible);
        this.txtSum.setVisibility(visible);
        this.btnSettings.setVisibility(visible);

        if(DataManager.getInstance().getIs_guest()) {
            this.btnCharge.setEnabled(false);
            this.btnCharge.setVisibility(View.INVISIBLE);
        } else {
            this.btnCharge.setEnabled(enable);
            this.btnCharge.setVisibility(visible);
        }
    }

    protected void refreshUserData(List<User> userList) {
        this.atxvAdapter.clear();
        for (User anUserList : userList) {
            this.atxvAdapter.add(anUserList);
        }
    }
    public void refreshProductData(List<Product> productList) {
        this.lstProductAdapter.clear();
        this.lstProductAdapter.addAll(productList);
    }

    protected void removeOrder(Order order) {
        this.lstOrderedAdapter.remove(order);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnCharge:
                this.chargeDialog = new ChargeDialog();
                this.chargeDialog.show();
                break;
            case R.id.btnPay:
                this.payChoiceDialog = new PayChoiceDialog();
                this.payChoiceDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialog) {
                        if (MainManager.getInstance().getCurrentUser() == null)
                            MainManager.getInstance().clearCurrentUser();
                    }
                });

                this.payChoiceDialog.show();
                break;
            case R.id.btnLogout:
                MainManager.getInstance().clearCurrentUser();
                break;
            case R.id.btnSettings:
                this.popupMenuSettings = new PopupMenu(this.mainActivity.getApplicationContext(), v);
                this.popupMenuSettings.setOnMenuItemClickListener(this.mainActivity);
                this.popupMenuSettings.inflate(R.menu.menu_main);
                if((!MainManager.getInstance().getCurrentUser().getCardId().equals("") || DataManager.getInstance().getIs_guest())) {
                    this.popupMenuSettings.getMenu().getItem(0).setEnabled(false);
                }
                this.popupMenuSettings.show();

                break;
            case R.id.btnRefresh:
                RefreshManager.getInstance().refreshData();
                break;
            case R.id.btnGuest:
                if(MainManager.getInstance().getCurrentUser() == null) {
                    DataManager.getInstance().setIs_guest(true);
                    MainManager.getInstance().setCurrentUser(new User("Guest", "", new BigDecimal(0), ""));
                    this.atxvName.setText("Gast");
                    this.atxvName.dismissDropDown();
                }
        }
    }


    //@ToDo: Custom Charge spinnt
    // @ToDo: payment dismiss spinnt
    @Override
    public boolean onKey(View v, int keyCode, KeyEvent event) {
        if (event.getAction() == KeyEvent.ACTION_UP) {
            if (KeyEvent.KEYCODE_ENTER == event.getKeyCode()) {
                if (this.atxvAdapter.getCount() == 1) {
                    MainManager.getInstance().setCurrentUser((User) this.atxvAdapter.getItem(0));
                    this.atxvName.setText(this.atxvAdapter.getItem(0).toString());
                    this.atxvName.dismissDropDown();
                    DataManager.getInstance().setIs_guest(false);
                }
                return false;

            }
        }
        return false;
    }
    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        switch (parent.getId()) {
            case -1:
                MainManager.getInstance().setCurrentUser((User) this.atxvAdapter.getItem(position));
            break;
        }
    }

    public void showCategoryButtons() {
        List<Category> categoryList = Category.getAll();
        this.gridCategories.removeAllViews();
        for (Category c: categoryList) {
            Button catButton = new Button(this.mainActivity.getApplicationContext());
            catButton.setText(c.getName());
            catButton.setHeight(120);
            catButton.setWidth(150);
            catButton.setGravity(0);
            catButton.setOnClickListener(new CategoryOnClickListener());
            this.gridCategories.addView(catButton);
        }
        for (int i=categoryList.size();i<12;i++) {
            Button btn = new Button(this.mainActivity.getApplicationContext());
            btn.setHeight(120);
            btn.setWidth(150);
            btn.setGravity(0);
            btn.setText("");
            btn.setEnabled(false);
            this.gridCategories.addView(btn);
        }
    }


    public void addOrder(Order order) {
        this.lstOrderedAdapter.insert(order, this.lstOrderedAdapter.getCount());
    }

    public void setProductDialog(ProductDialog productDialog) {
        this.productDialog = productDialog;
    }

    public void setChargeCustomDialog(ChargeCustomDialog chargeCustomDialog) {
        this.chargeCustomDialog = chargeCustomDialog;
    }

    public void showWarning(String message, int time) {
        this.showMessageBox(message, true, time);
    }

    public void showMessage(String message, int time) {
        this.showMessageBox(message, false, time);
    }

    private void showMessageBox(String message, boolean warning, int time) {
        NotificationDialog ndTransfer = new NotificationDialog(MainManager.getInstance().getMainActivity());
        if(warning)
            ndTransfer.setWarning();
        ndTransfer.setMessage(message);
        ndTransfer.show(time);
    }

}
